function Game() {
    this.$recordsPopup = $("#records_popup");
    this.$popupPointsParagraph = $("#points_paragraph");
    this.$popupPlayAgainBtn = $("#records_popup_btn");
    this.$popupPlayAgainBtn.click(function(e) {
        objects.game.$recordsPopup.addClass(domElements.invisibleClassName);
        objects.game.startNewGame();
        utils.turnOnScreen(domElements.$gameScreen);
    });

    this.$pointsSpan = $("#points_span");
    this.$livesSpan = $("#lives_span");
    this.$helpSpan = $("#help_span");
    this.$timeSpan = $("#time_span");

    this.levelPoints = this.pointsNeeded = this.totalPoints = this.time = undefined;

    this.$levelBlock = $("#level_passed_window");
    this.$statLevelPointsSpan = $("#level_points_stat_span");
    this.$statTotalPointsSpan = $("#total_points_stat_span");
    this.$statNextLevelPointsSpan = $("#next_points_stat_span");
    this.$okStatBtn = $("#stat_popup_btn");
    this.$okStatBtn.bind("click", function(e) {
        objects.game.$levelBlock.addClass(domElements.invisibleClassName);
        objects.game.startNewLevel();
    });
    this.pastSeconds = undefined;

    this.tick = this.timer = this.clearTimeout = this.pauseTimeout = undefined;
    this.background = this.player = this.elements = this.elementsNeeded =undefined;

    this.setView();
}

Game.prototype.setView = function() {
    this.background = new Background();
    this.player = new Player();
}

Game.prototype.setMouseClick = function(mousePoint) {
    if(mousePoint.y < settings.FISHES_START_Y || mousePoint.y > settings.FISHES_END_Y
            || typeof (this.player.targetPoint) != "undefined") return;
    this.player.setBobberPoint(mousePoint);
}

Game.prototype.startNewGame = function() {
    this.elements = new Array();
    this.levelPoints = 0;
    this.pointsNeeded = 0;
    this.totalPoints = 0;
    this.time = 0;
    this.elementsNeeded = 0;
    this.setTexts();
    this.tick = 0;
    this.startNewLevel();
    this.timer = window.setInterval(function() {
        objects.game.onFrame();
    }, constants.MILISECONDS_FOR_FRAME);
}

Game.prototype.startNewLevel = function() {
    this.levelPoints = 0;
    this.pastSeconds = new Date().getSeconds();
    this.time = settings.TIME_FOR_LEVEL;
    this.pointsNeeded = this.pointsNeeded == 0 ? settings.START_POINTS_NEEDED : this.pointsNeeded + settings.PLUS_LEVEL_POINTS_NEEDED;
    this.elementsNeeded = this.elementsNeeded == 0 ? settings.START_ONCE_ELEMENTS_COUNT : this.elementsNeeded - settings.MINUS_LEVEL_ONCE_ELEMENTS_COUNT;
    this.elementsNeeded = Math.max(this.elementsNeeded, settings.MIN_ONCE_ELEMENTS);
    this.player.setToStart();
    this.setTexts();
}

Game.prototype.endLevel = function() {
    if(this.levelPoints < this.pointsNeeded) {
        this.endGame();
        return;
    }

    this.$levelBlock.removeClass(domElements.invisibleClassName);
    this.$statLevelPointsSpan.text(this.levelPoints);
    this.$statTotalPointsSpan.text(this.totalPoints);
    this.$statNextLevelPointsSpan.text(this.pointsNeeded + settings.PLUS_LEVEL_POINTS_NEEDED);
}

Game.prototype.pauseGame = function(pauseMilliseconds) {
    if(typeof (this.timer) == "undefined") return;
    window.clearInterval(this.timer);
    this.timer = undefined;
    this.pauseTimeout = window.setTimeout(function(){objects.game.resumeGame();}, pauseMilliseconds);
}

Game.prototype.resumeGame = function() {
    window.clearTimeout(this.pauseTimeout);
    this.pauseTimeout = undefined;
    this.timer = window.setInterval(function() {objects.game.onFrame();}, constants.MILISECONDS_FOR_FRAME);
}

Game.prototype.onFrame = function() {
    this.tick += constants.MILISECONDS_FOR_FRAME;
    if(typeof (variables.pageX) != "undefined") this.setNewMovementPoint();
    if(this.tick % 1000 == 0) {
        var newSeconds = new Date().getSeconds();
        var difference = newSeconds - this.pastSeconds;
        if(difference < 0) difference += 60;
        this.pastSeconds = newSeconds;

        this.time -= difference;
        if(this.time < 0) this.time = 0;
        this.$timeSpan.text(this.time);
    }

    this.player.onFrame();

    if(this.time == 0 && typeof (this.player.goingDown) == "undefined") this.endLevel();

    if(Math.random() < settings.ELEMENT_SPAWN_PER_FRAME_POSSIBILITY / 100) {
        if(this.elements.length < this.elementsNeeded) this.spawnOneElement();
    }

    this.setElementsMovement();
	paper.view.draw();
}

Game.prototype.spawnOneElement = function() {
    var element = new Element();
    this.elements.push(element);
    $(element).bind(events.ELEMENT_OUT, function(e) {
        var index = objects.game.elements.indexOf(e.target);
        e.target.clear();
        objects.game.elements.splice(index, 1);
    });
}

Game.prototype.setElementsMovement = function() {
    var i = 0;
    while(i < this.elements.length) {
        this.elements[i].onFrame();
        if(typeof (this.player.goingDown) == "undefined" || !this.player.goingDown) {
            i++;
            continue;
        }
        if(this.elements[i].isCached(this.player.reactionCircle.position)) {
            this.player.setActiveElement(this.elements[i]);
            this.elements.splice(i, 1);
            continue;
        }
        i++;
    }
}

Game.prototype.setPlusPoints = function(points) {
    this.levelPoints += points;
    this.totalPoints += points;
    this.setTexts();
}

Game.prototype.endGame = function() {
    window.clearInterval(this.timer);
    this.timer = undefined;
    this.clearTimer = window.setTimeout(function() {
        objects.game.removeGameObjects();
    }, constants.MILISECONDS_FOR_FRAME * 2);
}

Game.prototype.removeGameObjects = function() {
    window.clearTimeout(this.clearTimer);
    this.clearTimer = undefined;

    variables.currentScore = this.totalPoints;
    this.player.clear();
    while(this.elements.length > 0) {
        this.elements[0].clear();
        this.elements.splice(0, 1);
    }

    if(settings.RECORDS_ENABLED) {
        objects.records.setNewRecords();
        utils.turnOnScreen(domElements.$recordScreen);
    }else {
        this.$recordsPopup.removeClass(domElements.invisibleClassName);
        this.$popupPointsParagraph.text(this.totalPoints);
    }
}

Game.prototype.setTexts = function() {
    this.$pointsSpan.text(this.levelPoints);
    this.$livesSpan.text(this.pointsNeeded);
    this.$helpSpan.text(this.totalPoints);
    this.$timeSpan.text(this.time);
}