function Player() {
    this.rope = this.targetPoint = this.startPoint = this.bobber = this.img = undefined;
    this.reactionCircle = this.goingDown = this.catchedFish = this.text = undefined;

    this.setPlayerImage();
    this.setBobberImage();
}

Player.prototype.setPlayerImage = function() {
    this.img = new paper.Raster(variables.playerImage);
    this.img.size.width = settings.PLAYER_WIDTH;
    this.img.size.height = variables.playerImage.height * (settings.PLAYER_WIDTH / variables.playerImage.width);
    this.img.position = new paper.Point(constants.GAME_WIDTH / 2, settings.PLAYER_Y_POSITION);
}

Player.prototype.setBobberImage = function() {
    this.startPoint = new paper.Point(constants.GAME_WIDTH / 2 + settings.BOBBER_X_OFFSET, this.img.position.y - settings.BOBBER_Y_OFFSET);
    this.bobber = new paper.Raster(variables.bobberImage);
    this.bobber.size.width = settings.BOBBER_WIDTH;
    this.bobber.size.height = variables.bobberImage.height * (settings.BOBBER_WIDTH / variables.bobberImage.width);
    this.bobber.position = this.startPoint;

    this.reactionCircle = new paper.Path.Circle(new paper.Point(this.bobber.position.x, this.bobber.position.y
            + this.bobber.size.height / 2), 10);
}

Player.prototype.setToStart = function() {
    this.bobber.position = this.startPoint;
    this.goingDown = undefined;
    this.targetPoint = undefined;
    this.bobber.matrix.rotation = 0;
    if(typeof (this.catchedFish) != "undefined") {
        objects.game.setPlusPoints(this.catchedFish.points);
        this.drawEarnedPoints(this.catchedFish.points);
        this.catchedFish.clear();
        this.catchedFish = undefined;
    }
    this.setReactionCircleToPlace();
    this.removeRope();
}

Player.prototype.drawEarnedPoints = function(points) {
    this.removeText();
    this.text = new paper.PointText({
        content: points,
        fillColor: settings.POINTS_TEXT_COLOR,
        fontFamily: 'Times new roman',
        fontWeight: 'bold',
        fontSize: settings.POINTS_TEXT_SIZE
    });
    this.text.position = this.bobber.position;
}

Player.prototype.setBobberPoint = function(mousePoint) {
    this.targetPoint = mousePoint;
    this.goingDown = true;
    this.setBobberRotation();
}

Player.prototype.setBobberRotation = function() {
    var deltaY = this.targetPoint.y - this.startPoint.y;
    var deltaX = this.targetPoint.x - this.startPoint.x;
    var degree = Math.atan2(deltaY, deltaX) * 180 / Math.PI;
    this.bobber.matrix.rotation = degree - 90;
    this.setReactionCircleToPlace();
}

Player.prototype.onFrame = function() {
    if(typeof (this.targetPoint) != "undefined") {
        if(this.goingDown) {
            this.goToTargetPoint();
        }else {
            this.goToStartPoint();
        }
    }

    if(typeof (this.text) != "undefined") {
        this.text.opacity -= 0.04;
        this.text.position.y -= 2;
        if(this.text.opacity < 0.1) this.removeText();
    }
}

Player.prototype.goToTargetPoint = function() {
    this.removeRope();
    var distance = this.bobber.position.getDistance(this.targetPoint);
    if(distance < settings.BOBBER_SPEED) {
        this.drawRope();
        this.bobber.position = this.targetPoint;
        this.goingDown = false;
        return;
    }

    var L = settings.BOBBER_SPEED / distance;
    var newActiveX = (1 - L) * this.bobber.position.x + L * this.targetPoint.x;
    var newActiveY = (1 - L) * this.bobber.position.y + L * this.targetPoint.y;
    this.bobber.position = new paper.Point(newActiveX, newActiveY);
    this.setReactionCircleToPlace();
    this.drawRope();
}

Player.prototype.goToStartPoint = function() {
    this.removeRope();
    var distance = this.bobber.position.getDistance(this.startPoint);
    var speed = typeof (this.catchedFish) == "undefined" ? settings.BOBBER_SPEED : settings.BOBBER_SPEED - this.catchedFish.minusSpeed;
    if(distance < speed) {
        this.setToStart();
        return;
    }

    var L = speed / distance;
    var newActiveX = (1 - L) * this.bobber.position.x + L * this.startPoint.x;
    var newActiveY = (1 - L) * this.bobber.position.y + L * this.startPoint.y;
    this.bobber.position = new paper.Point(newActiveX, newActiveY);
    this.setReactionCircleToPlace();
    if(typeof (this.catchedFish) != "undefined") this.setCachedFishToPosition();
    this.drawRope();
}

Player.prototype.setCachedFishToPosition = function() {
    this.catchedFish.img.matrix.rotation = 0;
//    this.catchedFish.img.position = new paper.Point(this.reactionCircle.position.x, this.reactionCircle.position.y + this.catchedFish.width / 2);
    this.catchedFish.img.position = this.reactionCircle.position;
    this.catchedFish.img.matrix.rotation = this.bobber.matrix.rotation - 90;
}

Player.prototype.setReactionCircleToPlace = function() {
    this.reactionCircle.matrix.rotation = 0;
    this.reactionCircle.position = new paper.Point(this.bobber.position.x, this.bobber.position.y
            + this.bobber.size.height / 2);
    this.reactionCircle.rotate(this.bobber.matrix.rotation, this.bobber.position);
}

Player.prototype.setActiveElement = function(element) {
    this.catchedFish = element;
    if(this.catchedFish.speed < 0)this.catchedFish.img.scale(-1, 1);
    this.goingDown = false;
    this.setCachedFishToPosition();
}

Player.prototype.drawRope = function() {
    this.rope = new paper.Path.Line(this.startPoint, this.bobber.position);
    this.rope.strokeColor = settings.ROPE_COLOR;
    this.rope.moveBelow(this.bobber);
}

Player.prototype.removeRope = function() {
    if(typeof (this.rope) != "undefined") {
        this.rope.remove();
        this.rope = undefined;
    }
}

Player.prototype.removeText = function() {
    if(typeof (this.text) != "undefined") {
        this.text.remove();
        this.text = undefined;
    }
}

Player.prototype.clear = function() {
    this.removeRope();
    this.removeText();
}